# Copyright 1999-2014 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2
# $Header: $

EAPI=5
inherit cvs eutils flag-o-matic toolchain-funcs

DESCRIPTION="A libc optimized for small size"
HOMEPAGE="http://www.fefe.de/dietlibc/"
ECVS_SERVER="cvs.fefe.de:/cvs"
ECVS_USER="cvs"
ECVS_MODULE="dietlibc"
ECVS_LOCALNAME="dietlibc"
ECVS_TOP_DIR="${DISTDIR}/cvs-src/${PN}" 
S="${WORKDIR}/dietlibc"

LICENSE="GPL-2"
SLOT="0"
KEYWORDS=""
IUSE=""

DEPEND=""
RDEPEND=""

DIETHOME=/usr/diet

src_prepare() {
	# Replace sparc64 related C[XX]FLAGS (see bug #45716)
	use sparc && replace-sparc64-flags

	# gcc-hppa suffers support for SSP, compilation will fail
	use hppa && strip-unsupported-flags

	# Makefile does not append CFLAGS
	append-flags -nostdinc -W -Wall -Wextra -Wchar-subscripts \
		-Wmissing-prototypes -Wmissing-declarations -Wno-switch \
		-Wno-unused -Wredundant-decls -fno-strict-aliasing

	# only use -nopie on archs that support it
	gcc-specs-pie && append-flags -nopie

	sed -i -e 's:strip::' Makefile || die
	append-flags -Wa,--noexecstack

	# add armv7 support
	sed -i 's/3-6/3-7/' Makefile
	perl -pi -e 's/\r\n/\n/g' arm/md5asm.S

	use arm && ln -sf bin-${ARCH} bin-${CHOST/-*/}
}

src_unpack() {
	cvs_src_unpack
}

src_compile() {
	emake prefix="${EPREFIX}"${DIETHOME} \
		CC="$(tc-getCC)" \
		CFLAGS="${CFLAGS}" \
		STRIP=":"
}

src_install() {
	emake prefix="${EPREFIX}"${DIETHOME} \
		DESTDIR="${D}" \
		install-bin \
		install-headers

	# broken so add a workaround                                                                                                                                                                                                         
	use arm && ln -sf lib-${ARCH} "${D}"/${DIETHOME}/lib-${CHOST/-*/}                                                                                                                                                                               

	dobin "${ED}"${DIETHOME}/bin/*
	doman "${ED}"${DIETHOME}/man/*/*
	rm -r "${ED}"${DIETHOME}/{man,bin} || die

	dodoc AUTHOR BUGS CAVEAT CHANGES README THANKS TODO PORTING
}

